#!/usr/bin/env python

import argparse
import os
import subprocess
import sys

def add_snmp_options(parser):
    parser.add_argument(
                        '-H', '--snmp_host',
                        help="host:port for SNMP",
                        default='127.0.0.1:161'
                        )
    parser.add_argument(
                        '-v', '--snmp_version',
                        choices=['', '1', '2', '3'],
                        help="specifies SNMP version to use",
                        default='2'
                        )
    parser.add_argument(
                        '-c', '--snmp_community',
                        help="set the community string",
                        default='public'
                        )
    parser.add_argument(
                        '-a', '--snmpv3_auth_protocol',
                        help="set authentication protocol (MD5|SHA)",
                        default=''
                        )
    parser.add_argument(
                        '-A', '--snmpv3_auth_passphrase',
                        help="set authentication protocol pass phrase",
                        default=''
                        )
    parser.add_argument(
                        '-x', '--snmpv3_priv_protocol',
                        help="set privacy protocol (DES|AES)",
                        default=''
                        )
    parser.add_argument(
                        '-X', '--snmpv3_priv_passphrase',
                        help="set privacy protocol pass phrase",
                        default=''
                        )
    parser.add_argument(
                        '-u', '--snmpv3_username',
                        help="set security name",
                        default=''
                        )
    parser.add_argument(
                        '-e', '--snmpv3_engine_id',
                        help='set security engine ID (e.g. 800073FA044452574543)',
                        default=''
                        )
    parser.add_argument(
                        '-n', '--snmpv3_context',
                        help='set context name (e.g. bridge1)',
                        default=''
                        )
    parser.add_argument(
                        '-T', '--snmp_timeout',
                        help='set the request timeout (in seconds)',
                        default=1
                        )


def get_snmp_options(args):
    result = []
    if   args.snmp_version == '1':
        result += ['-v', '1', '-c', args.snmp_community]

    elif args.snmp_version == '2':
        result += ['-v', '2c', '-c', args.snmp_community]

    else:
        result = ['-v', '3']

        auth = 'auth' if args.snmpv3_auth_passphrase else 'noAuth'
        priv = 'Priv' if args.snmpv3_priv_passphrase else 'NoPriv'
        result += ['-l', auth + priv]

        if args.snmpv3_username:
            result += ['-u', args.snmpv3_username]
        if args.snmpv3_auth_protocol:
            result += ['-a', args.snmpv3_auth_protocol]
        if args.snmpv3_auth_passphrase:
            result += ['-A', args.snmpv3_auth_passphrase]
        if args.snmpv3_priv_protocol:
            result += ['-x', args.snmpv3_priv_protocol]
        if args.snmpv3_priv_passphrase:
            result += ['-X', args.snmpv3_priv_passphrase]
        if args.snmpv3_engine_id:
            result += ['-e', args.snmpv3_engine_id]

    if args.snmp_timeout:
        result += ['-t', str(args.snmp_timeout)]

    return result + [args.snmp_host]

def snmpget(snmpoptions, debug, oids):
    cmdline = ['/bin/snmpget', '-Ovq'] + snmpoptions + list(oids)
    if debug:
        print('DEBUG: request:')
        for arg in cmdline: print(arg)

    try:
        result = subprocess.Popen(cmdline, stdout=subprocess.PIPE, stderr=subprocess.PIPE)

    except Exception, details:
        raise Exception(' '.join(cmdline) + ': ' + str(details))

    if result.wait():
        raise Exception(' '.join(cmdline) + ': ' + result.stderr.read())

    output = result.stdout.read()
    if debug: print("DEBUG: response:\n%s" % output)

    if not output:
        raise Exception(' '.join(cmdline) + ': ' + 'no response')

    return (resp for resp in output.split('\n') if resp)

def front_values(items):
    return (item[0] for item in items)

def back_values(items):
    return (item[-1] for item in items)

def sum_values_of_same_items(values, total_items):
    return [sum(values[i::total_items]) for i in range(total_items)]

def fetch_malware(do_fetch):
    items = (('mwKnownvirus'   , '.1.3.6.1.4.1.29690.2.2.1.1.0'),
             ('mwSuspicious'   , '.1.3.6.1.4.1.29690.2.2.1.2.0'),
             ('mwAdware'       , '.1.3.6.1.4.1.29690.2.2.1.3.0'),
             ('mwDialers'      , '.1.3.6.1.4.1.29690.2.2.1.4.0'),
             ('mwJoke'         , '.1.3.6.1.4.1.29690.2.2.1.5.0'),
             ('mwRiskware'     , '.1.3.6.1.4.1.29690.2.2.1.6.0'),
             ('mwHacktool'     , '.1.3.6.1.4.1.29690.2.2.1.7.0'))
	
    return zip(front_values(items), do_fetch(back_values(items)))

def fetch_filecheck(do_fetch):
    items = (('fcScannedFiles' , '.1.3.6.1.4.1.29690.2.3.1.3.101.0'),
             ('fcScannedBytes' , '.1.3.6.1.4.1.29690.2.3.1.3.102.0'),
             ('fcCacheHitFiles', '.1.3.6.1.4.1.29690.2.3.1.3.103.0'),
             ('fcScanErrors'   , '.1.3.6.1.4.1.29690.2.3.1.3.104.0'))

    return zip(front_values(items), do_fetch(back_values(items)))

def maild_speed(ifaces):
    items = (('mdPassed'       , '.1.3.6.1.4.1.29690.2.3.1.42.4.%d.1.0'),
             ('mdRepacked'     , '.1.3.6.1.4.1.29690.2.3.1.42.4.%d.2.0'),
             ('mdRejected'     , '.1.3.6.1.4.1.29690.2.3.1.42.4.%d.3.0'),
             ('mdFailed'       , '.1.3.6.1.4.1.29690.2.3.1.42.4.%d.4.0'),
             ('mdReceived'     , '.1.3.6.1.4.1.29690.2.3.1.42.4.%d.5.0'))

    def fetch_action(do_fetch):
        oids = [tpl % iface for iface in ifaces for tpl in back_values(items)]
        values = sum_values_of_same_items(map(int, do_fetch(oids)), len(items))
        values[-1] = sum(values) # TotalIn = QueueSize + TotalOut
        return zip(front_values(items), values)
	
    return fetch_action

def maild_queue(ifaces):
    items = (('mdQueueSize'    , '.1.3.6.1.4.1.29690.2.3.1.42.4.%d.5.0'),)

    def fetch_action(do_fetch):
        oids = [tpl % iface for iface in ifaces for tpl in back_values(items)]
        values = sum_values_of_same_items(map(int, do_fetch(oids)), len(items))
        return zip(front_values(items), values)

    return fetch_action
		
if __name__ == "__main__":

    commands = {
        'malware'           : fetch_malware,
        'filecheck'         : fetch_filecheck,
        'maildNative'       : maild_speed((1,)),
        'maildNativeQueue'  : maild_queue((1,)),
        'maildMilter'       : maild_speed((2,)),
        'maildMilterQueue'  : maild_queue((2,)),
        'maildSpamc'        : maild_speed((3,)),
        'maildSpamcQueue'   : maild_queue((3,)),
        'maildRspamc'       : maild_speed((4,)),
        'maildRspamcQueue'  : maild_queue((4,)),
        'maildTotal'        : maild_speed((1, 2, 3, 4)),
        'maildTotalQueue'   : maild_queue((1, 2, 3, 4)),
    }

    try:
        parser = argparse.ArgumentParser('drweb.py')
        parser.add_argument(
                            '-t', '--target',
                            choices=commands.keys(),
                            help="get graph meta-info for specialized target",
                            required=True
                            )
        parser.add_argument(
                            '-d', '--debug',
                            help="print debug info",
                            action='store_true',
                            default=False
                            )

        add_snmp_options(parser)
        args = parser.parse_args()
		
    except Exception, details:
        print('ERROR: ' + str(details))
        parser.print_help()
        sys.exit(1)

    try:
        target = commands[args.target];
        do_fetch = lambda oids: snmpget(get_snmp_options(args), args.debug, oids)
        response = target(do_fetch)
        print(' '.join([name + ':' + str(value) for name, value in response]))
        sys.exit(0)

    except Exception, details:
        print('ERROR: ' + str(details))
        sys.exit(1)
